<%
' TextFiler 1.2
' Written by Hunter Beanland
' http://www.geocities.com/hbeanland/
' hbeanland@yahoo.com.au

' TextFiler is a class which can read and write any single byte or unicode delimited files (CSV, Tab etc) or Fixed Width files.
' Full Microsoft Excel delimited rules are also adhered to:
' Text including commas & tabs encapsulated in double quotes are treated as 1 field ("text,text" = text,text)
' Double quotes used in encapsulation are stripped ("Text" = Text)
' 2 sets of double quotes are treated as 1 set of double quotes ("" = escaped ")
' Text in quotes can run over several lines
' Fields are outputed in a variable length single dimension array & inputed via the same.

Class TextFiler
	Public ErrorText
	Public VirtualFileName
	Public AbsoluteFileName
	Public UnicodeFile		'0 = ANSI*, -1 = Unicode
	Public FileReadWrite	'1 = Read*, 2 = Write, 8 = Append
	Public LineDelimiter	'vbCRLF = carriage return & line feed, vbLF = Line feed, etc
	Public Fields
	Public LineNumber
	Private TS
	Private ColWidths
	Private FieldDelimiter
	Private FileMode

	Public Property Let Delimiter(ByVal strDelimiter)
	'Delimiter to use: , = Comma*, vbTab = Tab, etc (sinlge char only) Not used for fixed width files
		FieldDelimiter = left(strDelimiter,1)
		FileMode = "D"
	End Property

	Public Property Get Delimiter
	'Return the delimiter
		Delimiter = FieldDelimiter
	End Property

	Public Property Let FieldWidths(ByVal strColWidths)
	'Char widths with comma seperator. ie "10,15,4,2,100". Not used for delimited files
	Dim ArrayIndex
		ColWidths = split(strColWidths,",")
		for ArrayIndex = 0 to ubound(ColWidths)
			if not isnumeric(ColWidths(ArrayIndex)) then ColWidths(ArrayIndex) = 0
			ColWidths(ArrayIndex) = cint(ColWidths(ArrayIndex))
		next
		FileMode = "F"
	End Property

	Public Property Get FieldWidths
	'Return array of widths
		FieldWidths = ColWidths
	End Property
	
	Private Sub Class_Initialize
		ErrorText = ""
		UnicodeFile = 0
		Delimiter = ","
		LineDelimiter = vbCRLF
		FieldWidths = "1"
		FileReadWrite = 1
		FileMode = "D"
	End Sub

	Public Function OpenFile(ByVal FileName)
		'Check file name and open the file
		Dim objFSO
		if instr(trim(FileName),":") = 2 then 
			AbsoluteFileName = trim(FileName)
		elseif instr(trim(FileName),".") = 1 then
			ErrorText = "Path must be full virtual or absolute (. and .. not allowed)"
			OpenFile = False
			exit Function
		else
			VirtualFileName = trim(FileName)
			AbsoluteFileName = Server.MapPath(trim(FileName))
		end if
		LineNumber = 0
		Set objFSO = Server.CreateObject("Scripting.FileSystemObject")
		if FileReadWrite = 1 then
			Set TS = objFSO.OpenTextFile(AbsoluteFileName, FileReadWrite, False, UnicodeFile)
		else
			Set TS = objFSO.OpenTextFile(AbsoluteFileName, FileReadWrite, True, UnicodeFile)
		end if	
		Set objFSO = Nothing
	End Function
	
	Public Function ReadLine
		'Read a line then split the fields into the Fields array
		Dim LineSource
		If TS.AtendOfStream then
			ErrorText = "End of File"
			ReadLine = False
			Exit Function
		end if
		LineNumber = LineNumber +1 
		on error resume next
		LineSource = trim(TS.ReadLine)
		if err.number <> 0 then
			ErrorText = "Read error: " & err.number & " " & err.Description
			ReadLine = False
			Exit Function		
		end if
		If FileMode = "D" and LineSource <> "" then
			'Delimited files can have a odd number of quotes which means the row spans more than 1 line
			while CountChar(LineSource, """") mod 2 <> 0 and Not TS.AtendOfStream
				LineSource = LineSource & vbLF & trim(TS.ReadLine)
				if err.number <> 0 then
					ErrorText = "Read error: " & err.number & " " & err.Description
					ReadLine = False
					Exit Function		
				end if
			wend
		end if
		on error goto 0
		if LineSource = "" then
			redim Fields(0)
		else
			If FileMode = "D" then
				SplitDelimiter(LineSource)
			Else
				SplitFixed(LineSource)
			End if	
		end if
		ReadLine = True
	End Function
	
	Public Function WriteLine(ByVal FieldArray)
		'Save the array of fields to the next line of the file
		Dim LineSource, intLength, ArrayIndex
		If FileMode = "D" then
			'Delimited
			for i = 0 to ubound(FieldArray,1)
				if isnull(FieldArray(i)) then FieldArray(i) = ""
				if LineSource <> "" then LineSource = LineSource & Delimiter
				if instr(1,FieldArray(i),"""") > 0 or instr(1,FieldArray(i),vbLF) >0 or instr(1,FieldArray(i),Delimiter) >0 then
					LineSource = LineSource & """" & FieldArray(i) & """"
				else
					LineSource = LineSource & FieldArray(i)
				end if	
			Next
		Else
			'Fixed Width
			if ubound(FieldArray,1) <> ubound(ColWidths,1) then
				ErrorText = "Size of array is not the same as the number of column widths supplied"
				WriteLine = False
				Exit Function		
			end if
			for ArrayIndex = 0 to ubound(FieldArray,1)
				if isnull(FieldArray(ArrayIndex)) then FieldArray(ArrayIndex) = ""
				intLength =  Len(FieldArray(ArrayIndex))
				if intLength > ColWidths(ArrayIndex) then 
					LineSource = LineSource & left(FieldArray(ArrayIndex),ColWidths(ArrayIndex))
				elseif intLength < ColWidths(ArrayIndex) then
					LineSource = LineSource & FieldArray(ArrayIndex) & string(ColWidths(ArrayIndex)-intLength," ")
				else
					LineSource = LineSource & FieldArray(ArrayIndex)
				end if	
			Next
		End if	
		on error resume next
		TS.Write(LineSource & LineDelimiter)
		if err.number <> 0 then
			ErrorText = "Write error: " & err.number & " " & err.Description
			WriteLine = False
			Exit Function		
		end if
		on error goto 0
		LineNumber = LineNumber +1 
		WriteLine = True
	End Function	
	
	Public Function SplitDelimiter(SourceLine)
	' Take the line from the source string and split it into an array. 
	Dim SourceIndex, SourceChar, OutputArray(), OutputField, OutputIndex, InQuotes
		InQuotes = false
		OutputIndex = 0
		OutputField = ""
		for SourceIndex = 1 to len(SourceLine)
			SourceChar = mid(SourceLine, SourceIndex, 1)
			if SourceChar = """" and InQuotes then 
				InQuotes = false
			elseif SourceChar = """" and not InQuotes then 
				InQuotes = true
			end if	
			if (SourceChar = FieldDelimiter and not InQuotes) or SourceIndex = len(SourceLine) then 
				if SourceIndex = len(SourceLine) and SourceChar <> FieldDelimiter then OutputField = OutputField & SourceChar
				if left(OutputField,1) = """" then OutputField = right(OutputField,len(OutputField)-1)
				if right(OutputField,1) = """" then OutputField = left(OutputField,len(OutputField)-1)
				OutputField = replace(OutputField,"""""","""")
				redim preserve OutputArray(OutputIndex)
				OutputArray(OutputIndex) = OutputField
				OutputField = ""
				OutputIndex = OutputIndex + 1
			else
				OutputField = OutputField & SourceChar
			end if
		next
		Fields = OutputArray
	End Function

	Public Function SplitFixed(SourceLine)
	'Chop the string into the Fields array with the sizes given in FieldWidths
	Dim SourceIndex, OutputArray(), OutputIndex
		SourceIndex = 1
		for OutputIndex = 0 to ubound(ColWidths,1)
			redim preserve OutputArray(OutputIndex)
			OutputArray(OutputIndex) = mid(SourceLine,SourceIndex,ColWidths(OutputIndex))
			SourceIndex = SourceIndex + ColWidths(OutputIndex)
		next
		Fields = OutputArray
	End Function
	
	Private Function CountChar(SourceLine, SearchChar)
	' Counts the number of times SearchChar occurs in SourceLine. Used by SplitDelimiter
		Dim SourceIndex, HitCount
		HitCount = 0
		for SourceIndex = 1 to len(SourceLine)
			if mid(SourceLine,SourceIndex,1) = SearchChar then HitCount = HitCount +1
		next
		CountChar = HitCount
	End Function

	Public Sub CloseFile
		'Close and destroy
		On Error Resume Next
		If isobject(TS) then
			TS.Close
			Set TS = Nothing 
		end if	
		On Error Goto 0
	End Sub

	Private Sub Class_Terminate
		'Make sure file is closed (to release file locks and resources)
		CloseFile
	End Sub	
End Class

%>